/**
 * The BlendMode enum defines how translucent cells are rendered.
 */
export declare enum BlendMode {
    /**
     * No blending.  Alpha is ignored.
     */
    None = 0,
    /**
     * Alpha blending.
     *
     * dstRGB = (srcRGB * srcA) + (dstRGB * (1-srcA))
     *
     * dstA = srcA + (dstA * (1-srcA))
     */
    Blend = 1,
    /**
     * Additive blending.
     *
     * dstRGB = (srcRGB * srcA) + dstRGB
     *
     * dstA = dstA
     */
    Add = 2
}

/**
 * Capitalizes the first letter of the input string.
 * @param str - The original input string.
 * @returns The capitalized string.
 */
export declare function capitalize(str: string): string;

export declare class Cell {
    readonly x: number;
    readonly y: number;
    charCode: number;
    fg: Color;
    bg: Color;
    dirty: boolean;
    blocked: boolean;
    blockedSight: boolean;
    explored: boolean;
    visible: boolean;
    pathId: number;
    g: number;
    h: number;
    prev: Cell | null;
    constructor(x: number, y: number, charCode?: string | number, fg?: Color, bg?: Color);
    setCharCode(charCode: number): void;
    setForeground(fg: Color): void;
    setBackground(bg: Color): void;
    setValue(charCode: string | number, fg?: Color, bg?: Color): boolean;
    drawCell(otherCell: Cell, blendMode?: BlendMode): void;
    private blendColors;
    private clamp;
}

export declare enum Chars {
    SMILEY = 1,
    INVERSE_SMILEY = 2,
    HEART = 3,
    DIAMOND = 4,
    CLUB = 5,
    SPADE = 6,
    BULLET = 7,
    INVERSE_BULLET = 8,
    LIGHT_SHADE = 176,
    MEDIUM_SHADE = 177,
    DARK_SHADE = 178,
    BOX_SINGLE_VERTICAL = 179,
    BOX_SINGLE_VERTICAL_AND_SINGLE_LEFT = 180,
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_LEFT = 185,
    BOX_DOUBLE_VERTICAL = 186,
    BOX_DOUBLE_DOWN_AND_DOUBLE_LEFT = 187,
    BOX_DOUBLE_UP_AND_DOUBLE_LEFT = 188,
    BOX_SINGLE_DOWN_AND_SINGLE_LEFT = 191,
    BOX_SINGLE_UP_AND_SINGLE_RIGHT = 192,
    BOX_SINGLE_HORIZONTAL_AND_SINGLE_UP = 193,
    BOX_SINGLE_HORIZONTAL_AND_SINGLE_DOWN = 194,
    BOX_SINGLE_VERTICAL_AND_SINGLE_RIGHT = 195,
    BOX_SINGLE_HORIZONTAL = 196,
    BOX_SINGLE_VERTICAL_AND_SINGLE_HORIZONTAL = 197,
    BOX_DOUBLE_UP_AND_DOUBLE_RIGHT = 200,
    BOX_DOUBLE_DOWN_AND_DOUBLE_RIGHT = 201,
    BOX_DOUBLE_HORIZONTAL_AND_DOUBLE_UP = 202,
    BOX_DOUBLE_HORIZONTAL_AND_DOUBLE_DOWN = 203,
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_RIGHT = 204,
    BOX_DOUBLE_HORIZONTAL = 205,
    BOX_DOUBLE_VERTICAL_AND_DOUBLE_HORIZONTAL = 206,
    BOX_SINGLE_UP_AND_SINGLE_LEFT = 217,
    BOX_SINGLE_DOWN_AND_SINGLE_RIGHT = 218,
    BLOCK_FULL = 219,
    BLOCK_BOTTOM_HALF = 220,
    BLOCK_LEFT_HALF = 221,
    BLOCK_RIGHT_HALF = 222,
    BLOCK_TOP_HALF = 223
}

/**
 * Colodore palette.
 * Recalculated palette for the VIC II (Commodore 64) by Pepto.
 * See: https://www.pepto.de/projects/colorvic/
 * See: https://lospec.com/palette-list/colodore
 */
export declare const ColodorePalette: {
    BLACK: number;
    WHITE: number;
    RED: number;
    CYAN: number;
    VIOLET: number;
    GREEN: number;
    BLUE: number;
    YELLOW: number;
    ORANGE: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_GRAY: number;
    GRAY: number;
    LIGHT_GREEN: number;
    LIGHT_BLUE: number;
    LIGHT_GRAY: number;
};

export declare type Color = number;

/**
 * Orignal CGA palette.
 * See: https://en.wikipedia.org/wiki/Color_Graphics_Adapter
 * See: https://lospec.com/palette-list/color-graphics-adapter
 */
export declare const Colors: {
    BLACK: number;
    WHITE: number;
    LIGHT_GRAY: number;
    DARK_GRAY: number;
    YELLOW: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_RED: number;
    LIGHT_GREEN: number;
    DARK_GREEN: number;
    LIGHT_CYAN: number;
    DARK_CYAN: number;
    LIGHT_BLUE: number;
    DARK_BLUE: number;
    LIGHT_MAGENTA: number;
    DARK_MAGENTA: number;
    ORANGE: number;
};

/**
 * Commodore 64 palette.
 * See: https://www.c64-wiki.com/wiki/Color
 * See: https://lospec.com/palette-list/commodore64
 */
export declare const Commodore64Palette: {
    BLACK: number;
    WHITE: number;
    RED: number;
    CYAN: number;
    VIOLET: number;
    GREEN: number;
    BLUE: number;
    YELLOW: number;
    ORANGE: number;
    BROWN: number;
    LIGHT_RED: number;
    DARK_GRAY: number;
    GRAY: number;
    LIGHT_GREEN: number;
    LIGHT_BLUE: number;
    LIGHT_GRAY: number;
};

/**
 * Calculates path between two points using Dijkstra's algorithm.
 *
 * @param source - Starting point, must have x and y properties.
 * @param dest - Destination point, must have x and y properties.
 * @param maxDist - Maximum distance to examine.
 * @returns Array of steps if destination found; undefined otherwise.
 */
export declare function computePath(map: Console_2, source: PointLike, dest: PointLike, maxDist: number): Cell[] | undefined;

declare class Console_2 {
    readonly width: number;
    readonly height: number;
    readonly grid: Cell[][];
    originX: number;
    originY: number;
    minX: number;
    maxX: number;
    minY: number;
    maxY: number;
    radius: number;
    clip?: Rect;
    constructor(width: number, height: number, blockedFunc?: (x: number, y: number) => boolean);
    clear(): void;
    getCell(x: number, y: number): Cell | undefined;
    getCharCode(x: number, y: number): number | undefined;
    drawChar(x: number, y: number, c: string | number, fg?: Color, bg?: Color): void;
    drawString(x: number, y: number, str: string, fg?: Color, bg?: Color): void;
    drawStringLine(x: number, y: number, line: string, fg?: Color, bg?: Color): void;
    drawCenteredString(x: number, y: number, str: string, fg?: Color, bg?: Color): void;
    drawMessage(x: number, y: number, message: Message, maxWidth: number): number;
    drawHLine(x: number, y: number, width: number, c: string | number, fg?: Color, bg?: Color): void;
    drawVLine(x: number, y: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawRect(x: number, y: number, width: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawBox(x: number, y: number, width: number, height: number, topChar: number, rightChar: number, bottomChar: number, leftChar: number, topLeftChar: number, topRightChar: number, bottomRightChar: number, bottomLeftChar: number, fg?: Color, bg?: Color): void;
    drawSingleBox(x: number, y: number, width: number, height: number, fg?: Color, bg?: Color): void;
    drawDoubleBox(x: number, y: number, width: number, height: number, fg?: Color, bg?: Color): void;
    fillRect(x: number, y: number, width: number, height: number, c: string | number, fg?: Color, bg?: Color): void;
    drawConsole(dstX: number, dstY: number, srcConsole: Console_2, srcX: number, srcY: number, srcWidth: number, srcHeight: number, blendMode?: BlendMode): void;
    drawCell(x: number, y: number, cell: Cell, blendMode?: BlendMode): void;
    setBlocked(x: number, y: number, blocked: boolean): void;
    setBlockedSight(x: number, y: number, blockedSight: boolean): void;
    isVisible(x: number, y: number): boolean;
    isBlocked(x: number, y: number): boolean;
    isBlockedSight(x: number, y: number): boolean;
    /**
     * Compute the FOV in an octant adjacent to the Y axis
     */
    private computeOctantY;
    /**
     * Compute the FOV in an octant adjacent to the X axis
     */
    private computeOctantX;
    computeFov(originX: number, originY: number, radius: number, opt_noClear?: boolean, opt_octants?: number): void;
    /**
     * All visible tiles are marked as explored.
     */
    updateExplored(): void;
}
export { Console_2 as Console }

export declare interface CrtOptions {
    scale: number;
    blur: number;
    curvature: number;
    chroma: number;
    vignette: number;
    scanlineWidth: number;
    scanlineIntensity: number;
}

export declare const DEFAULT_FONT: Font;

export declare class DefaultDialogRenderer {
    getState(terminal: Terminal, dialog: Dialog): DialogState;
    draw(term: Terminal, dialogState: DialogState): void;
}

/**
 * Deserializes a JSON string to an object graph.
 * Handles circular references and class instances.
 * @param str - The JSON string to deserialize.
 * @returns The deserialized object graph.
 */
export declare function deserialize(str: string): unknown;

export declare abstract class Dialog {
    readonly contentsRect: Rect;
    readonly title: string;
    constructor(contentsRect: Rect, title: string);
    abstract drawContents(console: Console_2, offset: Point): void;
    abstract handleInput(terminal: Terminal, offset: Point): boolean;
}

export declare interface DialogRenderer {
    getState(terminal: Terminal, dialog: Dialog): DialogState;
    draw(term: Terminal, dialogState: DialogState): void;
}

export declare class DialogState {
    readonly dialog: Dialog;
    readonly rect: Rect;
    readonly contentsOffset: Point;
    open: boolean;
    count: number;
    buffer?: Console_2;
    constructor(dialog: Dialog, rect: Rect, contentsOffset: Point);
}

export declare function fixBoxCells(con: Console_2): void;

export declare class Font {
    readonly url: string;
    readonly charWidth: number;
    readonly charHeight: number;
    readonly scale: number;
    constructor(url: string, charWidth: number, charHeight: number, scale?: number);
}

/**
 * The FovOctants constants provide bitmasks for various directions.
 *
 * @example
 * ```
 *     \ 4 | 3 /
 *      \  |  /
 *    5  \ | /  2
 *        \|/
 *   ------+-------
 *        /|\
 *    6  / | \  1
 *      /  |  \
 *     / 7 | 0 \
 *```
 */
export declare enum FovOctants {
    OCTANT_SOUTH_SOUTHEAST = 1,
    OCTANT_EAST_SOUTHEAST = 2,
    OCTANT_EAST_NORTHTHEAST = 4,
    OCTANT_NORTH_NORTHEAST = 8,
    OCTANT_NORTH_NORTHWEST = 16,
    OCTANT_WEST_NORTHEAST = 32,
    OCTANT_WEST_SOUTHWEST = 64,
    OCTANT_SOUTH_SOUTHWEST = 128
}

export declare enum FovQuadrants {
    QUADRANT_SOUTHEAST = 3,
    QUADRANT_EAST = 6,
    QUADRANT_NORTHEAST = 12,
    QUADRANT_NORTH = 24,
    QUADRANT_NORTHWEST = 48,
    QUADRANT_WEST = 96,
    QUADRANT_SOUTHWEST = 192,
    QUADRANT_SOUTH = 129
}

/**
 * Converts a color from HSV format to RGBA format.
 *
 * Based on: https://stackoverflow.com/a/17243070/2051724
 *
 * @param h - Hue (0.0 - 1.0).
 * @param s - Saturation (0.0 - 1.0).
 * @param v - Value (0.0 - 1.0).
 * @param a - Optional alpha (0.0 - 1.0).
 * @returns A 32-bit unsigned integer color.
 */
export declare function fromHsv(h: number, s: number, v: number, a?: number): Color;

/**
 * Creates a big-endian 32-bit RGBA color from red, green, and blue components.
 * @param r - Red (0-255).
 * @param g - Green (0-255).
 * @param b - Blue (0-255).
 * @param a - Optional alpha (0-255).
 * @returns A 32-bit unsigned integer color.
 */
export declare function fromRgb(r: number, g: number, b: number, a?: number): Color;

export declare function getFovQuadrant(dx: number, dy: number): FovQuadrants;

export declare class GUI {
    readonly terminal: Terminal;
    readonly renderer: DialogRenderer;
    readonly dialogs: DialogState[];
    constructor(terminal: Terminal, renderer?: DialogRenderer);
    add(dialog: Dialog): void;
    /**
     * Handles input for currently active dialog.
     * Returns true if the input was handled.
     * Returns false otherwise.
     */
    handleInput(): boolean;
    draw(): void;
}

/**
 * The Input class represents a physical input.
 * Example: keyboard key or mouse button.
 */
export declare class Input {
    down: boolean;
    downTime: number;
    repeat: boolean;
    repeatTime: number;
    downCount: number;
    upCount: number;
    constructor();
    setDown(down: boolean): void;
    update(time: number): void;
    /**
     * Returns true if the input is "pressed".
     * Pressed is a one time event when the input first goes down.
     * It then repeats on repeat delay.
     */
    isPressed(): boolean;
    /**
     * Returns true if the input is "clicked".
     * Clicked is a one time event when the input first goes up.
     */
    isClicked(): boolean;
}

export declare class InputSet<T> {
    readonly inputs: Map<T, Input>;
    clear(): void;
    get(key: T): Input;
    updateAll(time: number): void;
}

export declare enum Key {
    VK_BACKSPACE = "Backspace",
    VK_TAB = "Tab",
    VK_ENTER = "Enter",
    VK_SHIFT_LEFT = "ShiftLeft",
    VK_SHIFT_RIGHT = "ShiftRight",
    VK_CONTROL_LEFT = "ControlLeft",
    VK_CONTROL_RIGHT = "ControlRight",
    VK_ALT_LEFT = "AltLeft",
    VK_ALT_RIGHT = "AltRight",
    VK_PAUSE = "Pause",
    VK_CAPS_LOCK = "CapsLock",
    VK_ESCAPE = "Escape",
    VK_SPACE = "Space",
    VK_PAGE_UP = "PageUp",
    VK_PAGE_DOWN = "PageDown",
    VK_END = "End",
    VK_HOME = "Home",
    VK_LEFT = "ArrowLeft",
    VK_UP = "ArrowUp",
    VK_RIGHT = "ArrowRight",
    VK_DOWN = "ArrowDown",
    VK_INSERT = "Insert",
    VK_DELETE = "Delete",
    VK_0 = "Digit0",
    VK_1 = "Digit1",
    VK_2 = "Digit2",
    VK_3 = "Digit3",
    VK_4 = "Digit4",
    VK_5 = "Digit5",
    VK_6 = "Digit6",
    VK_7 = "Digit7",
    VK_8 = "Digit8",
    VK_9 = "Digit9",
    VK_SEMICOLON = "Semicolon",
    VK_EQUALS = "Equal",
    VK_A = "KeyA",
    VK_B = "KeyB",
    VK_C = "KeyC",
    VK_D = "KeyD",
    VK_E = "KeyE",
    VK_F = "KeyF",
    VK_G = "KeyG",
    VK_H = "KeyH",
    VK_I = "KeyI",
    VK_J = "KeyJ",
    VK_K = "KeyK",
    VK_L = "KeyL",
    VK_M = "KeyM",
    VK_N = "KeyN",
    VK_O = "KeyO",
    VK_P = "KeyP",
    VK_Q = "KeyQ",
    VK_R = "KeyR",
    VK_S = "KeyS",
    VK_T = "KeyT",
    VK_U = "KeyU",
    VK_V = "KeyV",
    VK_W = "KeyW",
    VK_X = "KeyX",
    VK_Y = "KeyY",
    VK_Z = "KeyZ",
    VK_CONTEXT_MENU = "ContextMenu",
    VK_NUMPAD0 = "Numpad0",
    VK_NUMPAD1 = "Numpad1",
    VK_NUMPAD2 = "Numpad2",
    VK_NUMPAD3 = "Numpad3",
    VK_NUMPAD4 = "Numpad4",
    VK_NUMPAD5 = "Numpad5",
    VK_NUMPAD6 = "Numpad6",
    VK_NUMPAD7 = "Numpad7",
    VK_NUMPAD8 = "Numpad8",
    VK_NUMPAD9 = "Numpad9",
    VK_NUMPAD_ENTER = "NumpadEnter",
    VK_MULTIPLY = "NumpadMultiply",
    VK_ADD = "NumpadAdd",
    VK_SEPARATOR = "NumpadSeparator",
    VK_SUBTRACT = "NumpadSubtract",
    VK_DECIMAL = "NumpadDecimal",
    VK_DIVIDE = "NumpadDivide",
    VK_F1 = "F1",
    VK_F2 = "F2",
    VK_F3 = "F3",
    VK_F4 = "F4",
    VK_F5 = "F5",
    VK_F6 = "F6",
    VK_F7 = "F7",
    VK_F8 = "F8",
    VK_F9 = "F9",
    VK_F10 = "F10",
    VK_F11 = "F11",
    VK_F12 = "F12",
    VK_F13 = "F13",
    VK_F14 = "F14",
    VK_F15 = "F15",
    VK_F16 = "F16",
    VK_F17 = "F17",
    VK_F18 = "F18",
    VK_F19 = "F19",
    VK_F20 = "F20",
    VK_F21 = "F21",
    VK_F22 = "F22",
    VK_F23 = "F23",
    VK_F24 = "F24",
    VK_NUM_LOCK = "NumLock",
    VK_SCROLL_LOCK = "ScrollLock",
    VK_COMMA = "Comma",
    VK_PERIOD = "Period",
    VK_SLASH = "Slash",
    VK_BACKQUOTE = "Backquote",
    VK_OPEN_BRACKET = "BracketLeft",
    VK_BACK_SLASH = "Backslash",
    VK_CLOSE_BRACKET = "BracketRight",
    VK_QUOTE = "Quote",
    VK_META = "OSLeft"
}

export declare class Keyboard {
    readonly keys: InputSet<Key>;
    /**
     * Creates a new keyboard module.
     *
     * @param el - DOM el to attach listeners.
     */
    constructor(el: HTMLElement);
    clear(): void;
    getKey(key: Key): Input;
    setKey(e: KeyboardEvent, state: boolean): void;
    updateKeys(time: number): void;
}

export declare function loadImage(url: string, callback: (img: Console_2) => void): void;

export declare function loadImage2x(url: string, callback: (img: Console_2) => void): void;

export declare class Message {
    readonly text: string | undefined;
    readonly fg?: number | undefined;
    readonly bg?: number | undefined;
    readonly children?: Message[] | undefined;
    readonly align?: MessageAlign | undefined;
    constructor(text: string | undefined, fg?: number | undefined, bg?: number | undefined, children?: Message[] | undefined, align?: MessageAlign | undefined);
    getWidth(): number;
    getHeight(): number;
}

export declare enum MessageAlign {
    LEFT = 0,
    CENTER = 1,
    RIGHT = 2
}

export declare class MessageDialog extends Dialog {
    readonly message: string | Message;
    constructor(title: string, message: string | Message);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal): boolean;
}

export declare class Mouse {
    readonly el: HTMLCanvasElement;
    readonly width: number;
    readonly height: number;
    readonly buttons: InputSet<number>;
    private prevX;
    private prevY;
    x: number;
    y: number;
    dx: number;
    dy: number;
    wheelDeltaX: number;
    wheelDeltaY: number;
    constructor(terminal: Terminal);
    private handleTouchEvent;
    private handleEvent;
    private handleWheelEvent;
    private updatePosition;
    update(time: number): void;
}

/**
 * PICO 8 palette.
 * See: https://pico-8.fandom.com/wiki/Palette
 * See: https://lospec.com/palette-list/pico-8
 */
export declare const Pico8Palette: {
    BLACK: number;
    DARK_BLUE: number;
    DARK_PURPLE: number;
    DARK_GREEN: number;
    BROWN: number;
    DARK_GRAY: number;
    LIGHT_GRAY: number;
    WHITE: number;
    RED: number;
    ORANGE: number;
    YELLOW: number;
    GREEN: number;
    BLUE: number;
    LAVENDER: number;
    PINK: number;
    LIGHT_PEACH: number;
};

export declare class Point implements PointLike {
    readonly x: number;
    readonly y: number;
    constructor(x: number, y: number);
}

export declare interface PointLike {
    readonly x: number;
    readonly y: number;
}

export declare class Rect {
    readonly x: number;
    readonly y: number;
    readonly width: number;
    readonly height: number;
    readonly left: number;
    readonly top: number;
    readonly x2: number;
    readonly y2: number;
    constructor(x: number, y: number, width: number, height: number);
    getCenter(): Point;
    intersects(other: Rect): boolean;
    contains(point: Point | Rect): boolean;
}

/**
 * Random number generator.
 *
 * LCG
 * https://stackoverflow.com/a/424445/2051724
 */
export declare class RNG {
    private m;
    private a;
    private c;
    private state;
    /**
     * Creates a new random number generator.
     *
     * @param seed - The integer seed.
     */
    constructor(seed?: number);
    nextInt(): number;
    /**
     * Returns a floating point number between 0.0 and 1.0.
     */
    nextFloat(): number;
    /**
     * Returns an integer in the range start (inclusive) to end (exclusive).
     * @param start - Lower bound, inclusive.
     * @param end - Upper bound, exclusive.
     */
    nextRange(start: number, end: number): number;
    chooseIndex(chances: number[]): number;
    chooseKey(chancesMap: Record<string, number>): string;
}

export declare class ScrollableMessageDialog extends Dialog {
    readonly message: Message;
    readonly messagesHeight: number;
    readonly scrollMax: number;
    readonly scrollbarHeight: number;
    scrollY: number;
    constructor(rect: Rect, title: string, message: Message);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal): boolean;
}

export declare class SelectDialog extends Dialog {
    readonly options: string[];
    readonly callback: (i: number) => void;
    private hoverIndex;
    constructor(title: string, options: string[], callback: (i: number) => void);
    drawContents(console: Console_2, offset: Point): void;
    handleInput(terminal: Terminal, offset: Point): boolean;
    isMouseOverOption(terminal: Terminal, offset: Point, index: number): boolean;
}

/**
 * Decorates a class to make serializable.
 * Any class with the `@serializable` decorator will be serialized and deserialized.
 * @param value - The TypeScript class to mark as serializable.
 */
export declare function serializable(value: {
    name: string;
}): void;

/**
 * Serializes a value to JSON.
 * Handles circular references and class instances.
 * @param obj - The root object to serialize.
 * @returns A string representation of the object graph.
 */
export declare function serialize(obj: unknown): string;

export declare class Terminal extends Console_2 {
    readonly canvas: HTMLCanvasElement;
    readonly font: Font;
    readonly crt?: CrtOptions;
    readonly maxFps?: number;
    readonly pixelWidth: number;
    readonly pixelHeight: number;
    readonly pixelScale: number;
    readonly keys: Keyboard;
    readonly mouse: Mouse;
    readonly gl: WebGLRenderingContext;
    readonly program: WebGLProgram;
    readonly positionAttribLocation: number;
    readonly textureAttribLocation: number;
    readonly fgColorAttribLocation: number;
    readonly bgColorAttribLocation: number;
    readonly positionsArray: Float32Array;
    readonly indexArray: Uint16Array;
    readonly textureArray: Float32Array;
    readonly foregroundUint8Array: Uint8Array;
    readonly foregroundDataView: DataView;
    readonly backgroundUint8Array: Uint8Array;
    readonly backgroundDataView: DataView;
    readonly positionBuffer: WebGLBuffer;
    readonly indexBuffer: WebGLBuffer;
    readonly textureBuffer: WebGLBuffer;
    readonly foregroundBuffer: WebGLBuffer;
    readonly backgroundBuffer: WebGLBuffer;
    readonly texture: WebGLTexture;
    readonly frameBufferTexture: WebGLTexture;
    readonly frameBuffer: WebGLFramebuffer;
    readonly crtProgram: WebGLProgram;
    readonly crtBlurLocation: WebGLUniformLocation;
    readonly crtCurvatureLocation: WebGLUniformLocation;
    readonly crtChromaLocation: WebGLUniformLocation;
    readonly crtScanlineWidthLocation: WebGLUniformLocation;
    readonly crtScanlineIntensityLocation: WebGLUniformLocation;
    readonly crtVignetteLocation: WebGLUniformLocation;
    readonly crtPositionLocation: number;
    readonly crtTexCoordLocation: number;
    readonly crtPositionBuffer: WebGLBuffer;
    readonly crtTexCoordBuffer: WebGLBuffer;
    private lastRenderTime;
    private renderDelta;
    fps: number;
    averageFps: number;
    update?: () => void;
    constructor(canvas: HTMLCanvasElement, width: number, height: number, options?: TerminalOptions);
    private handleResize;
    private getAttribLocation;
    private flush;
    isKeyDown(key: Key): boolean;
    isKeyPressed(key: Key): boolean;
    getKeyDownCount(key: Key): number;
    /**
     * Returns a standard roguelike movement key if pressed.
     * Implemented control systems:
     * 1) Numpad arrows
     * 2) VIM keys
     * 3) Normal arrows (4 directions only)
     * 4) Numpad 5 and '.' (period) for "wait"
     * If a key is pressed, returns the movement delta.
     * If no key is pressed, returns undefined.
     * See: http://www.roguebasin.com/index.php?title=Preferred_Key_Controls
     */
    getMovementKey(movementKeys?: Partial<Record<Key, Point>>): Point | undefined;
    private buildShader;
    /**
     * Initialize a texture and load an image.
     * When the image finished loading copy it into the texture.
     * @param url - The texture image URL.
     */
    private loadTexture;
    private render;
    private renderCrt;
    private requestAnimationFrame;
    private renderLoop;
}

export declare interface TerminalOptions {
    font?: Font;
    crt?: CrtOptions;
    maxFps?: number;
}

/**
 * Returns the input string wrapped to the maximum line length.
 * @param str - The original input string.
 * @param maxLength - The maximum length of a single line.
 * @returns Array of word wrapped lines.
 */
export declare function wordWrap(str: string, maxLength: number): string[];

export { }
